#ifndef __UNITS_H__
#define __UNITS_H__

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif /* TRUE */

#define PI 3.1415926535897932384626433832795
#define UNITS_NUM_FRAMES  72

#include <string.h>
#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "graphics.h"
#include "fftfile.h"
#include "math.h"

#define AI_STATE_STOP     1000
#define AI_STATE_RUN      1001
#define AI_STATE_ATTACK   1002
#define AI_STATE_LOOK     1003
#define AI_STATE_IGNORE   1004
#define AI_STATE_WAITLOOK 1005

extern Mix_Chunk *explode;
extern Mix_Chunk *shot;
extern int SMOOTH;
extern bool LOG;

typedef struct{
	std::string type;
	unsigned int priceBought;
}__cargo;

class Unit
{
public:
	Unit();
	~Unit();
	void Reset();
	void CleanUp();
	void Init(int sx, int sy, std::string img, std::string fpk);
	void Display(int sx, int sy);
	void StatusDisplay(int sx, int sy, int sw, int sh, TTF_Font *font);
	void SetSpeed(int sSpeed);
	void SetAngle(int sAngle);
	void SetX(int sx);
	void SetY(int sy);
	void SetAI(std::string ai);
	void SetAIState(int state);
	void Move(int sx, int sy);
	void SetMMR(Uint8 sr);
	void SetMMG(Uint8 sg);
	void SetMMB(Uint8 sb);
	void SetWarp(bool sw);
	void SetName(std::string sname);
	void SetZoom(float szoom);
	void SetMaxSpeed(int ms);
	void SetMaxWarp(int mw);
	void SetMaxSafeWarp(int msw);
	void SetTurnDelay(int td);
	void SetWarpTurnDelay(int tdw);
	void SetHullHitMax(int hhm);
	void SetShieldHitMax(int shm);
	void SetOverHeatMax(int ohm);
	void SetShootPoint(int sx, int sy);
	void SetCredits(unsigned int cred);
	void AdjustShootPoint();
	void ResetHits();
	void SetHullHits(int hh);
	void SetShieldHits(int sh);
	void PrevWeapon();
	void NextWeapon();
	void Rotate(int rotation);
	void EnableWarp();
	void DisableWarp();
	void AddShield(std::string shieldpic);
	void AddWeaponToSlot(int slotid, std::string fpk, std::string fn, int amount=0);
	void RemoveWeaponFromSlot(int slotid);
	void ReduceAmmo(int slotid);
	void AddWeaponBayToSlot(int slotid, std::string holdweapon, int holdamount);
	void RemoveWeaponBayFromSlot(int slotid);
	void AddCargoBayToSlot(int slotid, int units);
	void RemoveCargoBayFromSlot(int slotid);
	void AddToCargo(__cargo newcargo);
	void SortCargo();
	void RemoveFromCargo(int cargoindex);
	__ship_slot GetCurWeaponSlot();
	int GetWidth();
	int GetHeight();
	int GetImageWidth();
	int GetImageHeight();
	int GetDispImageW();
	int GetDispImageH();
	Uint8 GetMMR();
	Uint8 GetMMG();
	Uint8 GetMMB();
	int GetSpeed();
	int GetAngle();
	int GetX();
	int GetY();
	double GetDX();
	double GetDY();
	int GetNegX();
	int GetNegY();
	int GetSectorX();
	int GetSectorY();
	int GetMaxSpeed();
	int GetMaxWarp();
	int GetMaxSafeWarp();
	int GetTurnDelay();
	int GetWarpTurnDelay();
	int GetMaxHullHits();
	int GetHullHits();
	int GetMaxShieldHits();
	int GetShieldHits();
	int GetShootX();
	int GetShootY();
	unsigned int GetCredits();
	int GetCargoUnits();
	int GetCargoFill();
	float GetZoom();
	std::string GetCurWeaponFN();
	std::string GetCurWeaponFPK();
	std::string GetName();
	bool AtWarp();
	bool WarpEnabled();
	bool HasShield();
	void Hit();
	bool IsAIFiring();
	bool IsOverHeated();
	void Orbit();
	void Orbit(int sx, int sy, int rad);
	void SetOrbit(bool orbit);
	bool IsOrbiter();
	void FreeImages();
	void FPK_NewImages(std::string fpk, std::string imagename);
	void ReloadImages();
	void LoadImages(int start, int end);
	bool IsLoaded(int pic);
	bool ShieldIsLoaded(int pic);
	SDL_Surface *GetCurImage();
	void DisplayDispImage(int x, int y);
	void ZeroSlots();
	void FillSlots(__ship_slot *sls, int max);
	__ship_slot *GetSlots();
	__cargo *GetCargo();
	int GetNumSlots();
	int GetHeatSeekingBay();
	int GetHeatSeekingAmmoFill();
	int GetHeatSeekingAmmoSpace();
	void Colorize(Uint8 r, Uint8 g, Uint8 b);
	bitmask *mask[UNITS_NUM_FRAMES];
	bitmask *smask[UNITS_NUM_FRAMES];
	int radius, orbitx, orbity;
private:
	int angleinc;
	int angle;
	double angle2;
	int speed;
	std::string imgload;
	std::string usingfpk;
	SDL_Surface *image[UNITS_NUM_FRAMES];
	SDL_Surface *dispimg;
	int x;
	int y;
	double dx;
	double dy;
	std::string aiscript;
	Uint32 aiwait;
	int aistate;
	int aiwaitfor;
	Uint8 mmr;
	Uint8 mmg;
	Uint8 mmb;
	Uint8 cr;
	Uint8 cg;
	Uint8 cb;
	bool warp;
	bool warpenabled;
	bool orbiter;
	std::string name;
	float zoom;
	SDL_Surface *shield[UNITS_NUM_FRAMES];
	std::string shieldload;
	bool hasShield;
	bool hadShield;
	int maxspeed;
	int maxwarp;
	int maxsafewarp;
	int turndelay;
	int warpturndelay;
	int weaponcount;
	int curweapon;
	int overheatcount, overheatmax;
	Uint32 overheatwait;
	int hullhitmax, shieldhitmax, numshieldhits, numhullhits;
	int shootx, shooty, origshootx, origshooty;
	bool aifire;
	std::string curweaponfn;
	std::string curweaponfpk;
	__ship_slot slots[__MAXSHIPSLOTS];
	int numslots;
	unsigned int credits;
	int cargounits;
	int cargofill;
	__cargo cargo[1000];
};

class MapObject
{
public:
	MapObject() {}
	~MapObject();
	void Init(int sx, int sy, std::string img);
	void Display(int sx, int sy);
	void SetName(std::string n);
	void SetX(int sx);
	void SetY(int sy);
	void SetMMR(Uint8 sr);
	void SetMMG(Uint8 sg);
	void SetMMB(Uint8 sb);
	void SetZoom(float szoom);
	int GetWidth();
	int GetHeight();
	Uint8 GetMMR();
	Uint8 GetMMG();
	Uint8 GetMMB();
	std::string GetName();
	int GetX();
	int GetY();
	int GetNegX();
	int GetNegY();
	void FreeImage();
	void LoadImage();
	int GetSectorX();
	int GetSectorY();
	bool MapObject::IsLoaded();
private:
	std::string imgload;
	SDL_Surface *image;
	std::string name;
	int x;
	int y;
	Uint8 mmr;
	Uint8 mmg;
	Uint8 mmb;
	bool loaded;
	float zoom;
};

typedef struct{
	std::string type;
	std::string baytype;
	std::string holdweapon;
	std::string name;
	std::string desc;
	std::string imagefn;
	std::string fpk;
	unsigned int price;
	unsigned int baseprice;
	SDL_Surface *dispimage;
	int holdamount;
	int baseimport;
	int amount;
	int bayw;
	int bayl;
	int bayd;
	int cargounits;
}__shopitem;

typedef struct{
	__shopitem buyitems[30];
	int stock[30];
	int sold[30];
	int buynext;
}__shop;

class Station : public MapObject
{
public:
	Station();
	~Station();
	void AddItemToBuyShop(__shopitem item);
	void AddSellableCargo(__cargo cargo);
	void AllowHullRepairs(unsigned int hullRepairCost);
	void AllowShieldRepairs(unsigned int shieldRepairCost);
	void Dock(Unit *ship);
	void ChangeSupply();
	Uint32 GetTicks();
private:
	bool HRepairsAllowed;
	bool SRepairsAllowed;
	unsigned int hrepairCost;
	unsigned int srepairCost;
	__shop shop;
	__cargo cargosell[30];
	int cargonext;
	Uint32 ticks;
	unsigned int pm;
	bool sell;
	bool buy;
};

class Map
{
public:
	Map() {}
	~Map();
	void Init();
	void AddMainShip(Unit *object);
	void AddToObjects(MapObject *object);
	void AddToStations(Station *object);
	void AddToActiveObjects(Unit *object);
	void RemoveFromActiveObjects(int start);
	void AddToShootingObjects(Unit *object, int aishipindex);
	void RemoveFromShootingObjects(int j);
	void AddToAIShootingObjects(Unit *object);
	void RemoveFromAIShootingObjects(int j);
	void DestroyObjects();
	void DisplayMinimap(int x, int y, int shipx, int shipy, int range);
	void DisplayOnScreenObjects(int x, int y, int w, int h);
	void CheckSectors(int sectorx, int sectory);
	int CheckCollisions(Unit *a);
	int CheckAIUnitCollisions(Unit *a);
	int CheckShipCollisions(Unit *a);
	void SpawnBullet(int x, int y, int speed, int angle, std::string img, std::string fpk, int aishipindex=-1, int slotid=0);
	void SpawnAIBullet(int x, int y, int speed, int angle, std::string img, std::string fpk, bool target=false);
	void MoveBullets();
	void MoveAI(int sx, int sy);
	void SetZoom(float szoom);
	void AddToAnimations(StaticAnim *anim);
	void RemoveFromActiveAnims(int start);
	void AddToAIUnits(Unit *unit);
	void RemoveFromAIUnits(int start);
	void SpawnAnim(int index, int x, int y);
	void Dock(int x, int y);
	int GetAIUnitX(int index);
	int GetAIUnitY(int index);
	void SetTarget(int index);
	void DisplayAITarget(int sx, int sy, int sw, int sh, TTF_Font *font, int targetx, int targety);
	int Map::GetAIUnitImageW(int index);
	int Map::GetAIUnitImageH(int index);
	int GetObjCount();
	int GetAICount();
	Unit *mainship;
private:
	MapObject *objects[5000];
	Station *stations[500];
	Unit *actobjects[1000];
	Unit *sobjects[100];
	Unit *aisobjects[100];
	StaticAnim *anims[20];
	Unit *aiunits[100];
	int next;
	int stationnext;
	int actnext;
	int snext;
	int aisnext;
	int animnext;
	int activeanimnext;
	int aiunitnext;
	int target;
	int shootindex[100];
	Uint32 s_ticks[100];
	Uint32 ais_ticks[100];
	float zoom;
	StaticAnim *activeanims[20];
	bool activeanim[20];
};

#endif /* __UNITS_H__ */

